import 'package:app/event.dart';
import 'package:app/result.dart';

/// Build message to be posted on GitHub
String buildComment(Result result, Event event, String commitSha) {
  String comment = '## Package analysis results for commit $commitSha';
  comment +=
      '\n(version of [pana](https://pub.dev/packages/pana) used: ${result.panaVersion})';
  comment +=
      '\n\n* Health score is **${result.healthScore.toString()} / 100.0**';
  comment +=
      '\n* Maintenance score is **${result.maintenanceScore.toString()} / 100.0**';
  if (result.healthSuggestions.isNotEmpty ||
      result.maintenanceSuggestions.isNotEmpty) {
    comment += '\n\n### Issues';
  }
  if (result.healthSuggestions.isNotEmpty) {
    comment += '\n#### Health';
    result.healthSuggestions
        .forEach((Suggestion s) => comment += _stringSuggestion(s));
  }
  if (result.maintenanceSuggestions.isNotEmpty) {
    comment += '\n#### Maintenance';
    result.maintenanceSuggestions
        .forEach((Suggestion s) => comment += _stringSuggestion(s));
  }
  return comment;
}

String _stringSuggestion(Suggestion suggestion) {
  String str = '\n* ';
  if (suggestion.title != null || suggestion.loss != null) {
    str += '**';
    if (suggestion.title != null) str += '${suggestion.title} ';
    if (suggestion.loss != null) {
      str += '(${suggestion.loss.toString()} points)';
    }
    str += '**: ';
  }
  ;
  str += suggestion.description.replaceAll(RegExp(r'(\n)+'), '\n  * ');
  return str;
}

/// Process the output of the pana command and returns the [Result]
Result processOutput(Map<String, dynamic> output) {
  final dynamic scores = output['scores'];
  final String panaVersion = output['runtimeInfo']['panaVersion'];
  final double healthScore = scores['health'];
  final double maintenanceScore = scores['maintenance'];
  final List<Suggestion> maintenanceSuggestions = <Suggestion>[];
  final List<Suggestion> healthSuggestions = <Suggestion>[];
  final List<LineSuggestion> lineSuggestions = <LineSuggestion>[];

  if (output.containsKey('suggestions')) {
    final List<Map<String, dynamic>> suggestions =
        List.castFrom<dynamic, Map<String, dynamic>>(output['suggestions']);
    maintenanceSuggestions.addAll(_parseSuggestions(suggestions));
  }

  if (output.containsKey('health')) {
    final Map<String, dynamic> health = output['health'];
    if (health.containsKey('suggestions')) {
      final List<Map<String, dynamic>> suggestions =
          List.castFrom<dynamic, Map<String, dynamic>>(health['suggestions']);
      healthSuggestions.addAll(_parseSuggestions(suggestions));
    }
  }

  if (output.containsKey('dartFiles')) {
    final Map<String, dynamic> dartFiles = output['dartFiles'];
    for (final String file in dartFiles.keys) {
      final Map<String, dynamic> details = dartFiles[file];
      if (details.containsKey('codeProblems')) {
        final List<Map<String, dynamic>> problems =
            List.castFrom<dynamic, Map<String, dynamic>>(
                details['codeProblems']);
        lineSuggestions.addAll(
            problems.map((Map<String, dynamic> jsonObj) => LineSuggestion(
                  lineNumber: jsonObj['line'],
                  description: jsonObj['description'],
                  relativePath: jsonObj['file'],
                )));
      }
    }
  }

  return Result(
    panaVersion: panaVersion,
    maintenanceScore: maintenanceScore,
    healthScore: healthScore,
    maintenanceSuggestions: maintenanceSuggestions,
    healthSuggestions: healthSuggestions,
    lineSuggestions: lineSuggestions,
  );
}

List<Suggestion> _parseSuggestions(List<Map<String, dynamic>> list) => list
    .map((Map<String, dynamic> s) => Suggestion(
          description: s['description'],
          title: s['title'],
          loss: s['score'],
        ))
    .toList();
